﻿using System.Collections.Generic;
using System.Linq;

namespace NMag.Core
{
    public class EnergyMarket
    {
        public EnergyMarket() { }

        private EnergyMarket(EnergyMarket market)
        {
            FirmPowerLevel = market.FirmPowerLevel;
            FirmPowerPrice = market.FirmPowerPrice;
            SummerStart = market.SummerStart;
            SummerEnd = market.SummerEnd;

            Mode = market.Mode;
            DailyLoadCurve = new DailyLoadCurve(market.DailyLoadCurve);
            VariableLoadCurve = new VariableLoadCurve(market.VariableLoadCurve);

            PreferenceFunctions = market.PreferenceFunctions.Select(f => f.Clone()).ToList();
        }

        public enum DistributionMode
        {
            DailyLoad = 1,
            Vairable = 3,
            Constant = 4
        }

        public double FirmPowerLevel { get; set; } = 0;
        public double FirmPowerPrice { get; set; } = 0;
        public int SummerStart { get; set; } = 0;
        public int SummerEnd { get; set; } = 0;

        public DistributionMode Mode { get; set; } = DistributionMode.Constant;
        public DailyLoadCurve DailyLoadCurve { get; set; } = new DailyLoadCurve(new List<Point>());             // Curve code 1
        public VariableLoadCurve VariableLoadCurve { get; set; } = new VariableLoadCurve(new List<Point>());    // Curve code 3

        public List<PreferenceFunction> PreferenceFunctions { get; set; } = new List<PreferenceFunction>();

        public EnergyMarket Clone()
        {
            return new EnergyMarket(this);
        }

        public Result ValidateCurves()
        {
            if(PreferenceFunctions.Count == 0){ return new Result(true); }

            PreferenceFunction invalidFunction = PreferenceFunctions.FirstOrDefault(p => !p.PreferenceCurve.Validate().Succes);
            if(invalidFunction != null)
            {
                return new Result(false, $"Preference curve #{PreferenceFunctions.IndexOf(invalidFunction)} is invalid.");
            }

            List<int[]> periods = PreferenceFunctions.Select(p => new int[] { PreferenceFunctions.IndexOf(p), p.Start, p.End }).ToList();

            while(periods.Count > 1)
            {
                for (int i = 1; i < periods.Count; i++)
                {
                    if((periods[0][1] >= periods[1][1] && 
                        periods[0][1] <= periods[1][2]) ||
                        (periods[0][2] >= periods[1][1] &&
                        periods[0][2] <= periods[1][2]))
                    {
                        return new Result(false, $"Preference curves {periods[0][0]} and {periods[1][0]} overlap.");
                    }
                }

                periods.RemoveAt(0);
            }

            return new Result(true);
        }
    }
}
