﻿using System.Collections.Generic;
using System.Linq;

namespace NMag.Core
{
    public class JobControl
    {
        public JobControl() { }

        private JobControl(JobControl jobControl)
        {
            Name = jobControl.Name;

            FirstYear = jobControl.FirstYear;
            LastYear = jobControl.LastYear;
            FirmPowerLevel = jobControl.FirmPowerLevel;
            ReservoirLevel = jobControl.ReservoirLevel;

            SimulationType = jobControl.SimulationType;
            FirstPeriod = jobControl.FirstPeriod;
            LastPeriod = jobControl.LastPeriod;

            OutputSystemDataSummary = jobControl.OutputSystemDataSummary;
            OutputWaterBalanceControl = jobControl.OutputWaterBalanceControl;
            OutputPowerProductionStatistics = jobControl.OutputPowerProductionStatistics;
            OutputPowerCurtrailmentStatistics = jobControl.OutputPowerCurtrailmentStatistics;

            TextOutputLevel = jobControl.TextOutputLevel;
            ModuleOutput1 = jobControl.ModuleOutput1;
            ModuleOutput2 = jobControl.ModuleOutput2;
            ModuleOutput3 = jobControl.ModuleOutput3;

            OutputInBinary = jobControl.OutputInBinary;
            BinaryFileName = jobControl.BinaryFileName;

            TimeSeries = jobControl.TimeSeries.Select(s => s.Clone()).ToList();
        }

        public string Name { get; set; }

        public int FirstYear { get; set; }
        public int LastYear { get; set; }
        public int FirmPowerLevel { get; set; }             // GWh
        public double ReservoirLevel { get; set; }          // Ratio
        
        public int SimulationType { get; set; }
        public int FirstPeriod { get; set; }
        public int LastPeriod { get; set; }

        public bool OutputSystemDataSummary { get; set; }
        public bool OutputWaterBalanceControl { get; set; }
        public bool OutputPowerProductionStatistics { get; set; }
        public bool OutputPowerCurtrailmentStatistics { get; set; }
        
        public int TextOutputLevel { get; set; }
        public int ModuleOutput1 { get; set; }
        public int ModuleOutput2 { get; set; }
        public int ModuleOutput3 { get; set; }

        public bool OutputInBinary { get; set; }
        public string BinaryFileName { get; set; } = "";

        public List<TimeSeries> TimeSeries { get; set; } = new List<TimeSeries>();

        public string GetOutputControlString()
        {
            string outputControl = OutputSystemDataSummary ? "1," : "0,";
            outputControl += OutputWaterBalanceControl ? "1," : "0,";
            outputControl += OutputPowerProductionStatistics ? "1," : "0,";
            outputControl += OutputPowerCurtrailmentStatistics ? "1," : "0,";

            return outputControl;
        }

        public JobControl Clone()
        {
            return new JobControl(this);
        }
    }
}
