﻿using NMag.Core.Utilization;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace NMag.Core
{
    public class RunoffSeriesWriter
    {
        private const int NUMBER_SIZE = 12;

        public Result Write(RunoffSeries series, string fileName)
        {
            if(series == null || series.HydroCurve.Points.Count == 0) { return new Result(false, "No data to write."); }

            if (series.HydroCurve.Points.Count % 365 != 0)
            {
                return new Result(false, "Runoff series does not have complete years.");
            }

            if (File.Exists(fileName))
            {
                try
                {
                    File.Delete(fileName);
                }
                catch
                {
                    return new Result(false, $"Can not delete {fileName}.");
                }
            }

            StringBuilder output = new StringBuilder();

            output.AppendLine("(60X,I12,2F12.0/46(8F12.0/))");

            for (int y = 0; y < series.HydroCurve.Points.Count / 365; y++)
            {
                int year = series.HydroCurve.FirstYear + y;

                List<string> values = series.HydroCurve.Points
                    .GetRange(y * 365, 2)
                    .Select(p => p.Y.Format(3))
                    .Select(v => v.AddLead(' ', NUMBER_SIZE - v.Length))
                    .ToList();

                output.AppendLine(string.Join("", Enumerable.Repeat("0".AddLead(' ', NUMBER_SIZE - 1), 5)) + 
                    year.ToString().AddLead(' ', NUMBER_SIZE - 4) +
                    string.Join("", values));

                for (int i = 0; i < 46; i++)
                {
                    int firstIndex = y * 365 + i * 8 + 2;
                    int count = i < 45 ? 8 : 3;

                    values = series.HydroCurve.Points
                         
                        .GetRange(firstIndex, count)
                        .Select(p => p.Y.Format(3))
                        .Select(v => v.AddLead(' ', NUMBER_SIZE - v.Length))
                        .ToList();

                    output.AppendLine(string.Join("", values));
                }
            }

            File.WriteAllText(fileName, output.ToString(), Encoding.GetEncoding(1252));

            return new Result(true);
        }
    }
}
