﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace NMag.Core.Simulation
{
    public class SimulationResult
    {
        public SimulationResult(List<Module> modules, Project project)
        {
            ProjectName = project?.Name ?? "";
            FirmPowerPrice = project?.EnergyMarket.FirmPowerPrice ?? 0;

            foreach(Module module in modules)
            {
                ModuleResults.Add(new ModuleResult(module));
            }
        }

        public string ProjectName { get; private set; } = "";

        public List<DateTime> Periods { get; private set; } = new List<DateTime>();
        public List<double> FirmPowerProduction { get; private set; } = new List<double>();
        public List<double> TotalProduction { get; private set; } = new List<double>();
        public List<double> Cost { get; private set; } = new List<double>();

        public List<ModuleResult> ModuleResults { get; private set; } = new List<ModuleResult>();

        public double TotalProductionSum
        {
            get { return TotalProduction.Sum(); }
        }
        public double TotalProductionAverage
        {
            get { return Periods.Count == 0 ? 0 : TotalProductionSum / Periods.Count * 365; }
        }

        public double FirmPowerProductionSum
        {
            get { return FirmPowerProduction.Sum(); }
        }
        public double FirmPowerProductionAverage
        {
            get { return Periods.Count == 0 ? 0 : FirmPowerProductionSum / Periods.Count * 365; }
        }

        public double CostSum
        {
            get { return Cost.Sum(); }
        }
        public double CostAverage
        {
            get { return Periods.Count == 0 ? 0 : CostSum / Periods.Count * 365; }
        }

        public double FirmPowerPrice { get; private set; }
        public double NetBenefit
        {
            get { return FirmPowerPrice * FirmPowerProductionSum / 100 - CostSum; }
        }
    }
}
