﻿using System;
using System.Globalization;

namespace NMag.Core.Utilization
{
    public static class DateHelper
    {
        public static DateTime DayToDate(int year, int dayNumber)
        {
            if(year == 0 || dayNumber == 0)
            {
                return new DateTime(2000, 1, 1);
            }

            dayNumber--;
            dayNumber = Math.Max(dayNumber, 0);

            year += dayNumber / 365;
            dayNumber %= 365;

            if(DateTime.IsLeapYear(year) && dayNumber > 58)
            {
                dayNumber++;
            }

            return new DateTime(year, 1, 1).AddDays(dayNumber);
        }

        public static int DayOfYear(int firstYear, DateTime date)
        {
            return DayOfYear(new DateTime(1999, date.Month, date.Day)) + 365 * (date.Year - firstYear);
        }

        public static int DayOfYear(DateTime date)
        {
            return date.DayOfYear;
        }

        public static int DayOfYear(int month, int day)
        {
            return DayOfYear(new DateTime(1999, month, day));
        }

        public static bool ParseDate(string text, bool dayLast, out DateTime date)
        {
            text = text.LettersAndDigitsOnly();

            if(dayLast)
            {
                if (DateTime.TryParseExact(text, "yyyyMMdd", CultureInfo.InvariantCulture, DateTimeStyles.None, out date)) { return true; }
                if (DateTime.TryParseExact(text, "yyMMdd", CultureInfo.InvariantCulture, DateTimeStyles.None, out date)) { return true; }
                if (DateTime.TryParseExact(text, "MMMdd", CultureInfo.InvariantCulture, DateTimeStyles.None, out date)) { return true; }
                if (DateTime.TryParseExact(text, "MMdd", CultureInfo.InvariantCulture, DateTimeStyles.None, out date)) { return true; }
            }
            else
            {
                if (DateTime.TryParseExact(text, "ddMMyyyy", CultureInfo.InvariantCulture, DateTimeStyles.None, out date)) { return true; }
                if (DateTime.TryParseExact(text, "ddMMyy", CultureInfo.InvariantCulture, DateTimeStyles.None, out date)) { return true; }
                if (DateTime.TryParseExact(text, "ddMMM", CultureInfo.InvariantCulture, DateTimeStyles.None, out date)) { return true; }
                if (DateTime.TryParseExact(text, "ddMM", CultureInfo.InvariantCulture, DateTimeStyles.None, out date)) { return true; }
            }

            return false;
        }
    }
}
