﻿using System;
using System.Globalization;
using System.Linq;
using System.Text.RegularExpressions;

namespace NMag.Core.Utilization
{
    public static partial class Util
    {
        public static string DigitsOnly(this string text)
        {
            return new string((text ?? "").Where(c => char.IsDigit(c)).ToArray());
        }

        public static string LettersAndDigitsOnly(this string text)
        {
            return new string(text.Where(c => char.IsLetterOrDigit(c)).ToArray());
        }
        
        public static string AddLead(this string text, char lead, int count)
        {
            if (count < 0) { return text; }

            return new string(lead, count) + text;
        }

        public static string AddTail(this string text, char tail, int count)
        {
            if(count < 0) { return text; }

            return text + new string(tail, count);
        }

        public static string Left(this string text)
        {
            return text.Left(1);
        }

        public static string Left(this string text, int length)
        {
            if (text == null || length < 1)
            {
                return "";
            }

            return text.Substring(0, Math.Min(text.Length, length));
        }

        public static string Right(this string text)
        {
            return text.Right(1);
        }

        public static string Right(this string text, int length)
        {
            if (text == null || length < 1)
            {
                return "";
            }

            length = Math.Min(text.Length, length);

            return text.Substring(text.Length - length, length);
        }

        public static bool IsValidNumber(this string text)
        {
            return new Regex(@"^-?\d*\.?\d*$").IsMatch(text);
        }

        public static string RemoveTailingZeros(this string number)
        {
            int digitCount = 0;

            for (int c = number.Length - 1; c >= 0; c--)
            {
                if(number[c] != '0')
                {
                    digitCount = c + 1;
                    break;
                }
            }

            return number.Substring(0, digitCount);
        }

        public static string Format(this double value, int precision, bool useMinimum = false)
        {
            if(precision == 0)
            {
                return ((int)value).ToString($"D", CultureInfo.GetCultureInfo("en-US"));
            }

            if(precision < 0 || useMinimum)
            {
                string number = value.ToString();

                precision = !number.Contains('.') ?
                    Math.Max(precision, 0) :
                    Math.Max(1, number.Split('.')[1].RemoveTailingZeros().Count());

                return value.ToString($"F{precision}", CultureInfo.GetCultureInfo("en-US"));
            }

            return value.ToString($"F{precision}", CultureInfo.GetCultureInfo("en-US"));
        }
    }
}