﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;

namespace NMag.UI.CustomControls
{
    public class CustomDataGrid : DataGrid
    { 
        public static readonly DependencyProperty SelectedRowsProperty = DependencyProperty.Register(
            "SelectedRows",
            typeof(List<int>),
            typeof(CustomDataGrid));

        public List<int> SelectedRows
        {
            get
            {
                return GetValue(SelectedRowsProperty) as List<int>;
            }
            set
            {
                SetValue(SelectedRowsProperty, value);
            }
        }

        protected override void OnSelectionChanged(SelectionChangedEventArgs e)
        { 
            SelectedRows = GetSelectedRows();

            base.OnSelectionChanged(e);
        }

        private List<int> GetSelectedRows()
        {
            List<int> selectedIndices = new List<int>();

            if (ItemsSource == null)
            {
                return selectedIndices;
            }

            foreach (object item in ItemsSource)
            {
                if (ItemContainerGenerator.ContainerFromItem(item) is DataGridRow row)
                {
                    if(row.IsSelected)
                    {
                        selectedIndices.Add(row.GetIndex());
                    }
                }
            }

            selectedIndices.Sort();

            if(selectedIndices.Count == 0 && SelectedIndex > -1)
            {
                return new List<int>() { SelectedIndex };
            }

            return selectedIndices;
        }
    }
}
