﻿using NMag.UI.Miscellaneous;
using LiveCharts;
using System;
using NMag.UI.Windows;

namespace NMag.UI.CustomControls
{
    // TODO create factory
    public class ChartViewModel : ObservableObject
    {
        private Action _updateAction = null;

        public ChartViewModel(SeriesData seriesData)
        {
            SeriesData = seriesData;
        }

        public ChartViewModel(SeriesData seriesData, Action updateAction)
        {
            _updateAction = updateAction;
            SeriesData = seriesData;
        }

        public SeriesData SeriesData { get; set; }
        public bool HasCurvePoints
        {
            get { return SeriesData?.Curve == null ? false : SeriesData.Curve.Points.Count > 0; }
        }

        public bool CanEdit
        {
            get { return _updateAction != null; }
        }

        public bool IsTransposed
        {
            get { return SeriesData?.IsTransposed ?? false; }
            set { if (SeriesData != null) { SeriesData.IsTransposed = value; } }
        }

        public bool IsDateMode
        {
            get { return SeriesData?.IsDateMode ?? false; }
            set { if (SeriesData != null) { SeriesData.IsDateMode = value; } }
        }

        public string XAxisTitle
        {
            get { return SeriesData?.Curve?.Axes[IsTransposed ? 1 : 0].Title ?? ""; }
        }
        public string YAxisTitle
        {
            get { return SeriesData?.Curve?.Axes[IsTransposed ? 0 : 1].Title ?? ""; }
        }

        public Func<double, string> XFormatter
        {
            get { return SeriesData?.XFormatter; }
        }
        public Func<double, string> YFormatter
        {
            get { return SeriesData?.YFormatter; }
        }

        public SeriesCollection SeriesCollection
        {
            get { return new SeriesCollection { SeriesData.GetSeries() }; }
        }

        public RelayCommand ShowDataCommand
        {
            get
            {
                return new RelayCommand(args => ShowData(),
                    args => CanEdit);
            }
        }

        public RelayCommand TransposeCommand
        {
            get
            {
                return new RelayCommand(args => TransposeChart(),
                    args => SeriesData?.Curve?.CanTranspose ?? false);
            }
        }

        public RelayCommand ToggleDatesCommand
        {
            get
            {
                return new RelayCommand(args => ToggleDates(),
                    args => SeriesData?.Curve.IsTimeSeries ?? false);
            }
        }

        private void UpdateChart()
        {
            OnPropertyChanged("IsTransposed");
            OnPropertyChanged("IsDateMode");

            OnPropertyChanged("XAxisTitle");
            OnPropertyChanged("YAxisTitle");

            OnPropertyChanged("XFormatter");
            OnPropertyChanged("YFormatter");

            OnPropertyChanged("SeriesCollection");
        }

        private void TransposeChart()
        {
            if(!SeriesData?.Curve.CanTranspose ?? false) { return; }

            IsTransposed = !IsTransposed;
            UpdateChart();
        }

        private void ToggleDates()
        {
            if (!SeriesData?.Curve.IsTimeSeries ?? false) { return; }

            IsDateMode = !IsDateMode;
            UpdateChart();
        }

        private void ShowData()
        {
            DataViewerWindow window = new DataViewerWindow(SeriesData, _updateAction);
            window.ShowDialog();
        }
    }
}
