﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Interactivity;

namespace NMag.UI.CustomControls
{
    public class TextBoxUpdateOnLostKeyboardFocusBehavior : Behavior<TextBox>
    {
        protected override void OnAttached()
        {
            if (AssociatedObject != null)
            {
                base.OnAttached();
                AssociatedObject.LostKeyboardFocus += OnKeyboardLostFocus;
            }
        }

        protected override void OnDetaching()
        {
            if (AssociatedObject != null)
            {
                AssociatedObject.LostKeyboardFocus -= OnKeyboardLostFocus;
                base.OnDetaching();
            }
        }

        private void OnKeyboardLostFocus(object sender, KeyboardFocusChangedEventArgs e)
        {
            if (sender is TextBox textBox && e.NewFocus == null)
            {
                FrameworkElement parent = (FrameworkElement)textBox.Parent;
                while (parent is IInputElement && !((IInputElement)parent).Focusable)
                {
                    parent = (FrameworkElement)parent.Parent;
                }

                FocusManager.SetFocusedElement(FocusManager.GetFocusScope(textBox), parent);
            }
        }
    }
}
