﻿using System.Windows;

namespace NMag.UI.Dialogs
{
    /// <summary>
    /// Interaction logic for ModuleDialog.xaml
    /// </summary>
    public partial class ModuleDialog : BaseDialog
    {
        public ModuleDialog(string unit1Name = "", string unit2Name = "")
        {
            InitializeComponent();

            Title = (unit1Name == "") ?
                "Create New Module" :
                "Rename Module";

            ComboBox_Type.SelectedIndex = 0;

            if (unit1Name != "")
            {
                if(unit2Name != "")
                {
                    ComboBox_Type.SelectedIndex = 4;
                    TextBox_Unit2.Text = unit2Name;
                }

                TextBox_Unit1.Text = unit1Name;
                ComboBox_Type.Visibility = Visibility.Hidden;
            }            
        }

        private void ComboBox_Type_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            if(ComboBox_Type.SelectedIndex == 4)
            {
                Label_Unit1.Content = "Reservoir Name:";
                Label_Unit2.Content = "Power Plant Name:";
                Label_Unit2.Visibility = Visibility.Visible;
                TextBox_Unit2.Visibility = Visibility.Visible;
            }
            else
            {
                Label_Unit1.Content = "Name:";
                TextBox_Unit2.Text = "";
                Label_Unit2.Visibility = Visibility.Hidden;
                TextBox_Unit2.Visibility = Visibility.Hidden;
            }
        }

        public string[] Names
        {
            get { return new string[] { TextBox_Unit1.Text, TextBox_Unit2.Text.ToString()}; }
        }

        // TODO Use enum
        public int Type
        {
            get { return ComboBox_Type.SelectedIndex; }
        }

        protected override void On_Ok()
        {
            string error = "";

            if(ComboBox_Type.SelectedIndex == 4)
            {
                if (TextBox_Unit1.Text == "") { error = "Reservoir name not given."; }
                if (TextBox_Unit2.Text == "") { error = "Power Plant name not given."; }

                if (TextBox_Unit1.Text == TextBox_Unit2.Text) { error = "Names cannot be the same."; }
            }
            else
            {
                if ((TextBox_Unit1.Text ?? "") == "") { error = "Module name not given."; }
            }

            if (error != "")
            {
                MessageBox.Show(error, "Invalid Input", MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return;
            }

            base.On_Ok();
        }

        private void Button_Ok_Click(object sender, RoutedEventArgs e)
        {
            On_Ok();
        }

        private void Button_Cancel_Click(object sender, RoutedEventArgs e)
        {
            On_Cancel();
        }
    }
}
