﻿using NMag.Core;
using System;
using System.Windows.Input;

namespace NMag.UI.FlowDiagram
{
    public class ShapeFactory
    {
        private const int MODULE_WIDTH = 120;
        private const int MODULE_HEIGHT = 40;

        private MouseButtonEventHandler _onModuleClickHandler;

        public ShapeFactory(MouseButtonEventHandler onModuleClickHandler)
        {
            _onModuleClickHandler = onModuleClickHandler;
        }

        public enum ShapeType
        {
            Reservoir,
            PowerPlant,
            Transfer,
            ControlPoint,

            NumberShape
        }

        public ShapeBase CreateShape(Guid unitID, ShapeType shapeType, System.Windows.Point position, int width, int height)
        {
            switch(shapeType)
            {
                case ShapeType.Reservoir:
                    return new ReservoirShape(unitID, position, _onModuleClickHandler, width, height);

                case ShapeType.PowerPlant:
                    return new PowerPlantShape(unitID, position, _onModuleClickHandler, width, height);

                case ShapeType.Transfer:
                    return new TransferShape(unitID, position, _onModuleClickHandler, width, height);

                case ShapeType.ControlPoint:
                    return new ControlPointShape(unitID, position, _onModuleClickHandler, width, height);

                case ShapeType.NumberShape:
                    return new NumberShape(unitID, position, _onModuleClickHandler, width, height);

                default:

                    return null;
            }
        }

        public static ShapeType UnitTypeToShapeType(UnitType unitType)
        {
            switch(unitType)
            {
                case UnitType.Reservoir:
                    return ShapeType.Reservoir;

                case UnitType.PowerPlant:
                    return ShapeType.PowerPlant;

                case UnitType.Transfer:
                    return ShapeType.Transfer;

                case UnitType.ControlPoint:
                    return ShapeType.ControlPoint;

                default:
                    return ShapeType.NumberShape;
            }
        }
    }
}
