﻿using NMag.Core.Simulation;
using NMag.UI.Miscellaneous;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows.Input;

namespace NMag.UI.Views
{
    public class ResultsViewModel : MainViewModel
    {
        public ResultsViewModel(ApplicationViewModel application)
            :base(application, "Results", ViewModelGroup.Results)
        {

        }

        public override ToolbarViewModel Toolbar
        {
            get
            {
                return null;
            }
        }

        public ObservableCollection<string> Results
        {
            get { return new ObservableCollection<string>(GetResults()); }
        }

        public ICommand ExportResultsCommand
        {
            get
            {
                return new RelayCommand(args => ExportResults(),
                    args => Application.SimulationResult != null);
            }
        }

        public override void GetData()
        {
        }

        public override void SaveData()
        {
        }

        public override void Update()
        {
            UpdateToolbar();

            OnPropertyChanged("Results");
        }

        private List<string> GetResults()
        {
            if (Application.SimulationResult == null) { return new List<string>(); }

            ResultWriter writer = new ResultWriter();

            List<string> results = new List<string>();
            results.AddRange(writer.FormatOverview(Application.SimulationResult).Select(r => r.ToString(10)));
            results.AddRange(writer.FormatDetails(Application.SimulationResult).Select(r => r.ToString(10)));

            return results;
        }

        private void ExportResults()
        {
            new ResultWriter().Write(Application.SimulationResult, Application.ProjectName, "");
        }
    }
}
