﻿using NMag.Core;
using NMag.UI.Miscellaneous;
using System.Windows.Input;
using System.Windows.Media;

namespace NMag.UI.Views
{
    public class CanvasControls : ObservableObject
    {
        private WaterSystemViewModel _waterSystemViewModel;

        private bool _isMouseDown = false;
        private Point _dragStart = new Point();

        private Point _canvasSize = new Point();
        private Point _canvasCenter = new Point();

        private double _zoom = 1;

        private double _mouseX;
        private double _mouseY;

        public double MouseX
        {
            get { return _mouseX; }
            set { _mouseX = value; }
        }
        public double MouseY
        {
            get { return _mouseY; }
            set { _mouseY = value; }
        }

        public CanvasControls(WaterSystemViewModel waterSystemViewModel)
        {
            _waterSystemViewModel = waterSystemViewModel;

            MoveSystemTo(500, 500);
        }

        public MatrixTransform RenderTransform { get; set; } = new MatrixTransform();

        public bool CancelMove { get; set; }

        public void MoveSystemTo(double xCoordinate, double yCoordinate)
        {
            Matrix matrix = RenderTransform.Value;
            matrix.Translate(xCoordinate - _canvasCenter.Y, yCoordinate - _canvasCenter.Y);
            RenderTransform = new MatrixTransform(matrix);

            _canvasCenter = _canvasCenter.Add(new Point(xCoordinate, yCoordinate));

            _waterSystemViewModel.UpdateCanvas();
        }

        public void MoveSystemBy(double xDistance, double yDistance)
        {
            Matrix matrix = RenderTransform.Value;
            matrix.Translate(xDistance, yDistance);
            RenderTransform = new MatrixTransform(matrix);

            _canvasCenter = _canvasCenter.Add(new Point(xDistance, yDistance));

            _waterSystemViewModel.UpdateCanvas();
        }

        public void OnMouseMove()
        {
            if (CancelMove)
            {
                CancelMove = false;
                _isMouseDown = false;
                return;
            }

            if (!_isMouseDown) { return; }
            
            MoveSystemBy(MouseX - _dragStart.X, MouseY - _dragStart.Y);

            _dragStart = new Point(MouseX, MouseY);
        }

        public void OnMouseDown(MouseEventArgs args)
        {
            _isMouseDown = true;
            _dragStart = new Point(MouseX, MouseY);
        }

        public void OnMouseUp(MouseEventArgs args)
        {
            _isMouseDown = false;
        }

        public void OnZoom(object sender, int direction)
        {
            double scale = direction == 1 ? 1.2 : (1.0 / 1.2);

            _zoom *= scale;

            Matrix matrix = RenderTransform.Value;
            matrix.ScaleAt(scale, scale, MouseX, MouseY);
            RenderTransform = new MatrixTransform(matrix);

            _waterSystemViewModel.UpdateCanvas();
        }

        public void OnChangeCanvasSize(object canvasSize)
        {
            object[] size = canvasSize as object[];

            if (size == null
                || size.Length != 2
                || size[0].GetType() != typeof(double)
                || size[1].GetType() != typeof(double))
            { return; }
            
            double newWidth = (double)size[0];
            double newHeight = (double)size[1];

            MoveSystemBy((newWidth - _canvasSize.X) / 2, (newHeight - _canvasSize.Y) / 2);

            _canvasSize = new Point(newWidth, newHeight);
        }
    }
}
