﻿using NMag.Core;
using NMag.UI.CustomControls;
using System;

namespace NMag.UI.Views
{
    public class NMagStrategyViewModel : OperationalStrategyViewModel
    {
        private string _annualRelase;

        private int _strategy;

        public NMagStrategyViewModel(ModuleSetupViewModel moduleViewModel)
            : base(moduleViewModel)
        {
            GetData();
        }

        public int Strategy
        {
            get { return _strategy; }
            set 
            {
                _strategy = value;
                Update();
            }
        }

        public bool UseReleaseCurve
        {
            get { return Strategy == (int)NMagOperationalStrategy.OperationalStrategy.ReservoirRelease; }
        }
        public string AnnualRelease
        {
            get { return _annualRelase; }
            set { _annualRelase = value; }
        }
        public ChartViewModel ReleaseChart
        {
            get
            {
                if (OperationalStrategy?.ReservoirReleaseCurve == null) { return null; }

                return new ChartViewModel(
                    new SeriesData(
                        OperationalStrategy.ReservoirReleaseCurve,
                        SeriesData.SeriesType.Line),
                    Update);
            }
        }

        public bool UseGuideCurve
        {
            get { return Strategy == (int)NMagOperationalStrategy.OperationalStrategy.ReservoirGuide; }
        }
        public ChartViewModel GuideChart
        {
            get
            {
                if (OperationalStrategy?.ReservoirGuideCurve == null) { return null; }

                return new ChartViewModel(
                    new SeriesData(
                        OperationalStrategy?.ReservoirGuideCurve,
                        SeriesData.SeriesType.Line),
                    Update);
            }
        }

        private NMagOperationalStrategy OperationalStrategy
        {
            get { return _moduleViewModel.SelectedModule?.NMagOperationalStrategy; }
        }

        public override void GetData()
        {
            if(OperationalStrategy == null) { return; }

            AnnualRelease = OperationalStrategy.AnnualRelease.ToString() ?? "";
            Strategy = (int)OperationalStrategy.Strategy;
        }

        public override void SaveData()
        {
            if (OperationalStrategy == null) { return; }

            if (double.TryParse(AnnualRelease, out double value)) { OperationalStrategy.AnnualRelease = value; }

            OperationalStrategy.Strategy = (NMagOperationalStrategy.OperationalStrategy)Strategy;
        }

        public override void Update()
        {
            OnPropertyChanged("Strategy");

            OnPropertyChanged("AnnualRelease");
            OnPropertyChanged("UseReleaseCurve");
            OnPropertyChanged("ReleaseChart");

            OnPropertyChanged("UseGuideCurve");
            OnPropertyChanged("GuideChart");
        }
    }
}
