﻿using NMag.Core;
using NMag.Core.Utilization;
using NMag.UI.CustomControls;
using System.Collections.Generic;
using System.Linq;

namespace NMag.UI.Views
{
    public class RoutingViewModel : ModuleDataViewModel
    {
        public RoutingViewModel(ModuleSetupViewModel moduleViewModel)
            : base(moduleViewModel) { }

        public List<string> UnselectedModules
        {
            get
            {
                if(_moduleViewModel.WaterSystem.Application.Project == null) { return null; }

                return new List<string>() { ModuleHelper.GetModuleAddress(null) }
                    .Concat(_moduleViewModel.WaterSystem.Application.Project.Modules
                        .Where(m => _moduleViewModel.SelectedModule == null ||
                            m.Number != _moduleViewModel.SelectedModule.Number)
                        .OrderBy(m => m.Number)
                        .Select(m => ModuleHelper.GetModuleAddress(m)))
                    .ToList();
            }
        }

        public bool IsReservoirUnit
        {
            get { return _moduleViewModel.SelectedUnit.UnitType == UnitType.Reservoir; }
        }

        public string SelectedRelease
        {
            get
            {
                return ModuleHelper.GetModuleAddress(_moduleViewModel.WaterSystem.Application.Project?.Modules
                    .FirstOrDefault(m => m.Number == _moduleViewModel.SelectedModule?.ReleaseModuleNumber));
            }
            set
            {
                if (_moduleViewModel.WaterSystem.Application.Project == null || _moduleViewModel.SelectedModule == null || (value ?? "") == "") { return; }

                Module sink = _moduleViewModel.WaterSystem.Application.Project.Modules
                    .FirstOrDefault(m => ModuleHelper.GetModuleAddress(m) == value);

                _moduleViewModel.SelectedModule.ReleaseModuleNumber = (sink == null) ? 0 : sink.Number;

                if (SelectedBypass.Left() == "0") { SelectedBypass = value; }
                if (SelectedSpill.Left() == "0") { SelectedSpill = value; }

                _moduleViewModel.WaterSystem.Application.Update();
            }
        }
        public string SelectedBypass
        {
            get
            {
                return ModuleHelper.GetModuleAddress(_moduleViewModel.WaterSystem.Application.Project?.Modules
                    .FirstOrDefault(m => m.Number == _moduleViewModel.SelectedModule?.BypassModuleNumber));
            }
            set
            {
                if (_moduleViewModel.WaterSystem.Application.Project == null || _moduleViewModel.SelectedModule == null || (value ?? "") == "") { return; }

                Module bypass = _moduleViewModel.WaterSystem.Application.Project.Modules
                    .FirstOrDefault(m => ModuleHelper.GetModuleAddress(m) == value);

                _moduleViewModel.SelectedModule.BypassModuleNumber = (bypass == null) ? 0 :bypass.Number;

                _moduleViewModel.WaterSystem.Application.Update();
            }
        }
        public string SelectedSpill
        {
            get
            {
                return ModuleHelper.GetModuleAddress(_moduleViewModel.WaterSystem.Application.Project?.Modules
                    .FirstOrDefault(m => m.Number == _moduleViewModel.SelectedModule?.SpillModuleNumber));
            }
            set
            {
                if (_moduleViewModel.WaterSystem.Application.Project == null || _moduleViewModel.SelectedModule == null || (value ?? "") == "") { return; }

                Module spill = _moduleViewModel.WaterSystem.Application.Project.Modules
                    .FirstOrDefault(m => ModuleHelper.GetModuleAddress(m) == value);

                _moduleViewModel.SelectedModule.SpillModuleNumber = (spill == null) ? 0 : spill.Number;

                _moduleViewModel.WaterSystem.Application.Update();
            }
        }

        public bool EnableRoutingCurve
        {
            get { return _moduleViewModel.SelectedModule?.EnableRoutingCurve ?? false; }
            set
            {
                if(_moduleViewModel.SelectedModule == null) { return; }

                _moduleViewModel.SelectedModule.EnableRoutingCurve = value;

                OnPropertyChanged("EnableRoutingCurve");
                OnPropertyChanged("RoutingChart");
            }
        }
        public ChartViewModel RoutingChart
        {
            get
            {
                if(_moduleViewModel.SelectedModule?.RoutingCurve == null) { return null; }

                return new ChartViewModel(
                    new SeriesData(
                        _moduleViewModel.SelectedModule?.RoutingCurve,
                        SeriesData.SeriesType.Column),
                    Update);
            }
        }

        public override void GetData()
        {

        }

        public override void SaveData()
        {

        }

        public override void Update()
        {
            OnPropertyChanged("UnselectedModules");

            OnPropertyChanged("SelectedRelease");
            OnPropertyChanged("SelectedBypass");
            OnPropertyChanged("SelectedSpill");

            OnPropertyChanged("EnableRoutingCurve");
            OnPropertyChanged("RoutingChart");
        }
    }
}
