﻿using NMag.Core;
using NMag.UI.CustomControls;

namespace NMag.UI.Views
{
    public class PowerPlantViewModel : ViewModel
    {
        private PowerPlantUnit _powerPlant;

        private string _maxDischarge;
        private string _energyEquivalent;

        private bool _usePeakingCurve;
        private bool _useOptionalData;

        private string _nominalHead;
        private string _intakeLevel;
        private string _tailWaterLevel;
        private string _headLossCoefficient;
        
        public PowerPlantViewModel(PowerPlantUnit powerPlant)
        {
            _powerPlant = powerPlant;

            GetData();
        }

        public string MaxDischarge
        {
            get { return _maxDischarge; }
            set { _maxDischarge = value; }
        }
        public string EnergyEquivalent
        {
            get { return _energyEquivalent; }
            set { _energyEquivalent = value; }
        }
         
        public bool UsePeakingCurve
        {
            get { return _usePeakingCurve; }
            set
            {
                _usePeakingCurve = value;
                OnPropertyChanged("UsePeakingCurve");
                OnPropertyChanged("PeakingChart");
            }
        }
        public ChartViewModel PeakingChart
        {
            get
            {
                if (_powerPlant?.PeakingCurve == null) { return null; }


                return new ChartViewModel(
                    new SeriesData(
                        _powerPlant?.PeakingCurve, 
                        SeriesData.SeriesType.Line), 
                    Update);
            }
        }

        public bool UseOptionalData
        {
            get { return _useOptionalData; }
            set
            {
                _useOptionalData = value;
                Update();
            }
        }
        public string NominalHead
        {
            get { return _nominalHead; }
            set { _nominalHead = value; }
        }
        public string IntakeLevel
        {
            get { return _intakeLevel; }
            set { _intakeLevel = value; }
        }
        public string TailWaterLevel
        {
            get { return _tailWaterLevel; }
            set { _tailWaterLevel = value; }
        }
        public string HeadLossCoefficient
        {
            get { return _headLossCoefficient; }
            set { _headLossCoefficient = value; }
        }

        public ChartViewModel LoadEfficiencyChart
        {
            get
            {
                if (_powerPlant?.LoadEfficiencyCurve == null) { return null; }


                return new ChartViewModel(
                    new SeriesData( 
                        _powerPlant?.LoadEfficiencyCurve, 
                        SeriesData.SeriesType.Line), 
                    Update);
            }
        }

        public override void SaveData()
        {
            if(_powerPlant == null) { return; }

            if (double.TryParse(MaxDischarge, out double value)) { _powerPlant.MaxDischarge = value; }
            if (double.TryParse(EnergyEquivalent, out value)) { _powerPlant.EnergyEquivalent = value; }

            _powerPlant.UsePeakingCurve = UsePeakingCurve;
            _powerPlant.UseOptionalData = UseOptionalData;

            if (double.TryParse(NominalHead, out value)) { _powerPlant.NominalHead = value; }
            if (double.TryParse(IntakeLevel, out value)) { _powerPlant.IntakeLevel = value; }
            if (double.TryParse(TailWaterLevel, out value)) { _powerPlant.TailWaterLevel = value; }
            if (double.TryParse(HeadLossCoefficient, out value)) { _powerPlant.HeadLossCoefficient = value; }
        }

        public override void GetData()
        {
            if (_powerPlant == null) { return; }

            MaxDischarge = _powerPlant.MaxDischarge.ToString();
            EnergyEquivalent = _powerPlant.EnergyEquivalent.ToString();

            UsePeakingCurve = _powerPlant.UsePeakingCurve;
            UseOptionalData = _powerPlant.UseOptionalData;

            NominalHead = _powerPlant.NominalHead.ToString();
            IntakeLevel = _powerPlant.IntakeLevel.ToString();
            TailWaterLevel = _powerPlant.TailWaterLevel.ToString();
            HeadLossCoefficient = _powerPlant.HeadLossCoefficient.ToString();
        }

        public override void Update()
        {
            OnPropertyChanged("MaxDischarge");
            OnPropertyChanged("EnergyEquivalent");

            OnPropertyChanged("UsePeakingCurve");
            OnPropertyChanged("PeakingChart");

            OnPropertyChanged("UseOptionalData");
            OnPropertyChanged("NominalHead");
            OnPropertyChanged("IntakeLevel");
            OnPropertyChanged("LoadEfficiencyChart");
            OnPropertyChanged("HeadLossCoefficient");
        }
    }
}
