﻿using NMag.Core;
using NMag.UI.CustomControls;

namespace NMag.UI.Views
{
    public class TransferViewModel : ViewModel
    {
        private TransferUnit _transfer;

        private int _mode;

        private string _capacity;

        private string _inletElevation;
        private string _outletElevation;
        private string _headLossCoefficient;

        private string _intakeEleavtion;
        private string _tailWaterElevation;

        public TransferViewModel(TransferUnit transfer)
        {
            _transfer = transfer;

            GetData();
        }

        public int Mode
        {
            get { return _mode; }
            set
            {
                _mode = value;
                Update();
            }
        }

        public bool UseFixedCapacity
        {
            get { return _transfer == null ? false : _mode == (int)TransferUnit.TransferMode.FixedCapacity; }
        }
        public string Capacity
        {
            get { return _capacity; }
            set { _capacity = value; }
        }

        public bool UseVariableCapacity
        {
            get { return _transfer == null ? false : _mode == (int)TransferUnit.TransferMode.VariableCapacity; }
        }
        public string InletElevation
        {
            get { return _inletElevation; }
            set { _inletElevation = value; }
        }
        public string OutletElevation
        {
            get { return _outletElevation; }
            set { _outletElevation = value; }
        }
        public string HeadLossCoefficient
        {
            get { return _headLossCoefficient; }
            set { _headLossCoefficient = value; }
        }

        public bool UseHQCapacityCurve
        {
            get { return _transfer == null ? false : _mode == (int)TransferUnit.TransferMode.HQCapacityCurve; }
        }
        public string IntakeElevation
        {
            get { return _intakeEleavtion; }
            set { _intakeEleavtion = value; }
        }
        public string TailwaterElevation
        {
            get { return _tailWaterElevation; }
            set { _tailWaterElevation = value; }
        }
        public ChartViewModel HQCapacityChart
        {
            get
            {
                if (_transfer?.HQCapacityCurve == null) { return null; }

                return new ChartViewModel(
                    new SeriesData(
                        _transfer.HQCapacityCurve, 
                        SeriesData.SeriesType.Line), 
                    Update);
            }
        }

        public override void SaveData()
        {
            if(_transfer == null) { return; }

            _transfer.Mode = (TransferUnit.TransferMode)_mode;

            if (double.TryParse(Capacity, out double value)) { _transfer.Capacity = value; }

            if (double.TryParse(InletElevation, out value)) { _transfer.InletElevation = value; }
            if (double.TryParse(OutletElevation, out value)) { _transfer.OutletElevation = value; }
            if (double.TryParse(HeadLossCoefficient, out value)) { _transfer.HeadLossCoefficient = value; }

            if (double.TryParse(IntakeElevation, out value)) { _transfer.IntakeElevation = value; }
            if (double.TryParse(TailwaterElevation, out value)) { _transfer.TailwaterElevation = value; }
        }

        public override void GetData()
        {
            if (_transfer == null) { return; }

            _mode = (int)_transfer.Mode;

            Capacity = _transfer.Capacity.ToString();

            InletElevation = _transfer.InletElevation.ToString();
            OutletElevation = _transfer.OutletElevation.ToString();
            HeadLossCoefficient = _transfer.HeadLossCoefficient.ToString();

            IntakeElevation = _transfer.IntakeElevation.ToString();
            TailwaterElevation = _transfer.TailwaterElevation.ToString();
        }

        public override void Update()
        {
            OnPropertyChanged("Mode");

            OnPropertyChanged("UseFixedCapacity");
            OnPropertyChanged("Capacity");

            OnPropertyChanged("UseVariableCapacity");
            OnPropertyChanged("InletElevation");
            OnPropertyChanged("OutletElevation");
            OnPropertyChanged("HeadLossCoefficient");

            OnPropertyChanged("UseHQCapacityCurve");
            OnPropertyChanged("IntakeElevation");
            OnPropertyChanged("TailwaterElevation");
            OnPropertyChanged("HQCapacityChart");
        }
    }
}